/// This is the simplest possible client using rustls that does something useful:
/// it accepts the default configuration, loads some root certs, and then connects
/// to google.com and issues a basic HTTP request.  The response is printed to stdout.
///
/// It makes use of rustls::Stream to treat the underlying TLS connection as a basic
/// bi-directional stream -- the underlying IO is performed transparently.
///
/// Note that `unwrap()` is used to deal with networking errors; this is not something
/// that is sensible outside of example code.
use std::{os::unix::net::SocketAddr, sync::Arc};

use log::{info, trace};
use std::net::ToSocketAddrs;
use tokio::{
    io::{self, split, AsyncReadExt, AsyncWriteExt, ReadHalf, WriteHalf},
    net::TcpStream,
};
use tokio_rustls::{
    rustls::{ClientConfig, KeyLogFile, RootCertStore},
    TlsConnector,
};

async fn run() -> io::Result<()> {
    let mut root_store = RootCertStore::empty();
    root_store.extend(webpki_roots::TLS_SERVER_ROOTS.iter().cloned());
    let mut config = ClientConfig::builder()
        .with_root_certificates(root_store)
        .with_no_client_auth();

    // Allow using SSLKEYLOGFILE.
    config.key_log = Arc::new(KeyLogFile::new());
    let connector = TlsConnector::from(Arc::new(config));
    let server_address = ("teilgedanken.de", 4190).to_socket_addrs()?.next().unwrap();
    let stream = TcpStream::connect(&server_address).await?;
    let domain = pki_types::ServerName::try_from("teilgedanken.de").unwrap();
    let stream = connector.connect(domain, stream).await?;
    let (mut reader, mut writer) = split(stream);
    let mut dst = String::new();

    reader
        .read_to_string(&mut dst)
        .await
        .expect("Failed to read");
    info!("{:?}", &dst);
    //writer.write(src)
    /* {
        let mut reader = BufReader::new(stream);
        let mut read_all = String::new();
        /*     loop {
            trace!("reading max {} bytes", buffer.len());
            let bytes_read = rea.read(&mut buffer);
            if let Ok(bytes_read) = bytes_read {
                trace!("read {:?} bytes", bytes_read);
                if bytes_read == buffer.len() {
                    let cur = String::from_utf8_lossy(&buffer[..bytes_read]);
                    read_all += &cur;
                    trace!("Received: \n{}", cur);
                } else {
                    let cur = String::from_utf8_lossy(&buffer[..bytes_read]);
                    read_all += &cur;
                    trace!("Last: \n{}", cur);
                    break;
                }
            } else {
                trace!("Error after reading continue");
                break;
            }
        } */
        reader.read_to_string(&mut read_all);
        info!("Read all:\n{}", read_all);
    }
    let ciphersuite = stream.conn.negotiated_cipher_suite().unwrap();
    writeln!(
        &mut std::io::stderr(),
        "Current ciphersuite: {:?}",
        ciphersuite.suite()
    )
    .unwrap();
    /*     stream
        .clone()
        .write_all(concat!("UNAUTHENTICATE\r\n",).as_bytes())
        .unwrap();
    loop {
        let bytes_read = stream.read(&mut buffer);
        if let Ok(bytes_read) = bytes_read {
            if bytes_read > 0 {
                trace!(
                    "Received: {:?}",
                    String::from_utf8_lossy(&buffer[..bytes_read])
                );
            } else {
                break;
            }
        } else {
            break;
        }
    } */ */
    Ok(())
}
fn main() {
    env_logger::init();
    tokio::runtime::Builder::new_current_thread()
        .enable_io()
        .build()
        .unwrap()
        .block_on(run())
        .expect("Failed to run or interrupted");
}
